/*
 * Copyright (c) 2016-2017, Parallels International GmbH
 *
 * This file is part of disp-helper. It's free software; you can redistribute
 * it and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 *
 * Our contact details: Parallels International GmbH, Vordergasse 59, 8200
 * Schaffhausen, Switzerland.
 */

#include <curl/curl.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#define CAPATH "/usr/share/disp-helper/"
#define CACERT "/usr/share/disp-helper/ovz-cert.crt"

struct buffer
{
	char *data;
	size_t len;
};

size_t write_function(void *data_, size_t size_, size_t nmemb_, struct buffer *s_)
{
	size_t n = size_ * nmemb_;
	size_t new_len = s_->len + n;
	s_->data = realloc(s_->data, new_len + 1);
	if (s_->data == NULL)
		return 0;
	memcpy(s_->data + s_->len, data_, n);
	s_->data[new_len] = '\0';
	s_->len = new_len;

	return n;
}

const char* protocol(void)
{
	return "CURL";
}

int sender(const char* file_, const char* server_, char** response_)
{
	CURL* hash = NULL;
	struct curl_httppost* post = NULL;
	struct curl_httppost* last = NULL;
	struct buffer s = { NULL, 0 };
	struct buffer certca = { NULL, 0 };
	CURLcode r;
	certca.data = strdup(CAPATH);
	s.data = strdup("");

	hash = curl_easy_init();

	curl_easy_setopt(hash, CURLOPT_URL, server_);
	curl_formadd(&post, &last,
		CURLFORM_COPYNAME, "name",
		CURLFORM_FILE, file_,
		CURLFORM_END
	);
	curl_easy_setopt(hash, CURLOPT_HTTPPOST, post);
	curl_easy_setopt(hash, CURLOPT_WRITEFUNCTION, write_function);
	curl_easy_setopt(hash, CURLOPT_WRITEDATA, &s);
	if(access(CACERT, F_OK) != -1) {
		curl_easy_setopt(hash, CURLOPT_CAPATH, certca.data);
	}
	/*curl_easy_setopt(hash, CURLOPT_VERBOSE, 1L);*/

    r = curl_easy_perform(hash);
    if(r != CURLE_OK) {
		if (s.data != NULL)
			free(s.data);
		return -r;
    }
	if (*response_ != NULL)
		free(*response_);
	*response_ = strdup(s.data);
	free(s.data);
	curl_formfree(post);
	curl_easy_cleanup(hash);

	return 0;
}

